@@ -5,7 +5,7 @@ use warnings;
 package Types::Capabilities;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.002001';
+our $VERSION   = '0.003000';
 
 use Type::Library -base;
 
@@ -20,6 +20,14 @@ my $_munge_handler = sub {
 	return ref( $handler )->new( %$handler, prefer_shift_self => 0 );
 };
 
+my @standard_generator_args = (
+	generator_for_slot         => sub { return '${$_[0]}' },
+	generator_for_get          => sub { return '${$_[0]}' },
+	generator_for_set          => sub { return '@${$_[0]} = @{' . pop . '}'; },
+	generator_for_default      => sub { return '[]'; },
+	get_is_lvalue              => !!1,
+);
+
 BEGIN {
 	# I don't like doing this, but curried arguments are breaking things...
 
@@ -35,7 +43,7 @@ BEGIN {
 		return Sub::HandlesVia::Handler->new(
 			name      => 'Array:peekend',
 			args      => 0,
-			template  => '($GET)->[0]',
+			template  => '($GET)->[-1]',
 		);
 	};
 
@@ -66,16 +74,14 @@ do {
 				name    => $cap_name,
 				methods => [ sort keys %$cap_methods ],
 				autobox => $array_class,
+				library => __PACKAGE__,
 			)
 		);
 
 		my $cg = Sub::HandlesVia::CodeGenerator->new(
 			target                     => $array_class,
-			generator_for_slot         => sub { return '$_[0]' },
-			generator_for_get          => sub { return '$_[0]' },
-			generator_for_set          => sub { return '$_[0] = ' . pop; },
-			generator_for_default      => sub { return '[]'; },
-			generator_for_usage_string => sub { "\$@{[ $lc_cap_name ]}->@{[ $_[1] ]}(@{[ $_[2] ]})" }
+			generator_for_usage_string => sub { "\$@{[ $lc_cap_name ]}->@{[ $_[1] ]}(@{[ $_[2] ]})" },
+			@standard_generator_args,
 		);
 
 		for my $method ( sort keys %$cap_methods ) {
@@ -119,16 +125,14 @@ do {
 				name    => $cap_name,
 				methods => [ sort keys %$cap_methods ],
 				autobox => $array_class,
+				library => __PACKAGE__,
 			)
 		);
 
 		my $cg = Sub::HandlesVia::CodeGenerator->new(
 			target                     => $array_class,
-			generator_for_slot         => sub { return '$_[0]' },
-			generator_for_get          => sub { return '$_[0]' },
-			generator_for_set          => sub { return '$_[0] = ' . pop; },
-			generator_for_default      => sub { return '[]'; },
-			generator_for_usage_string => sub { "\$@{[ $lc_cap_name ]}->@{[ $_[1] ]}(@{[ $_[2] ]})" }
+			generator_for_usage_string => sub { "\$@{[ $lc_cap_name ]}->@{[ $_[1] ]}(@{[ $_[2] ]})" },
+			@standard_generator_args,
 		);
 
 		for my $method ( sort keys %$cap_methods ) {
@@ -162,16 +166,14 @@ do {
 				name    => $cap_name,
 				methods => [ sort keys %$cap_methods ],
 				autobox => $array_class,
+				library => __PACKAGE__,
 			)
 		);
 		
 		my $cg = Sub::HandlesVia::CodeGenerator->new(
 			target                     => $array_class,
-			generator_for_slot         => sub { return '$_[0]' },
-			generator_for_get          => sub { return '$_[0]' },
-			generator_for_set          => sub { return '$_[0] = ' . pop; },
-			generator_for_default      => sub { return '[]'; },
-			generator_for_usage_string => sub { "\$@{[ $lc_cap_name ]}->@{[ $_[1] ]}(@{[ $_[2] ]})" }
+			generator_for_usage_string => sub { "\$@{[ $lc_cap_name ]}->@{[ $_[1] ]}(@{[ $_[2] ]})" },
+			@standard_generator_args,
 		);
 		
 		for my $method ( sort keys %$cap_methods ) {
@@ -186,9 +188,6 @@ do {
 };
 
 __PACKAGE__->make_immutable;
-
-1;
-
 __END__
 
 =pod
@@ -201,6 +200,10 @@ Types::Capabilities - don't care what type of data you are given, just what you
 
 =head1 SYNOPSIS
 
+Using B<Greppable> in a method signature:
+
+  use experimental qw( signatures );
+  
   package LineFilter {
     use Moo;
     use Type::Params 'signature_for';
@@ -225,6 +228,43 @@ Types::Capabilities - don't care what type of data you are given, just what you
   my $greetings = LineFilter->new( regexp => qr/Hello/ );
   $greetings->print_matching_lines( [ 'Hello world', 'Goodbye' ] );
 
+Using B<Enqueueable> and B<Dequeueable> in an attribute type constraint:
+
+  use experimental qw( signatures try );
+  
+  package TaskQueue {
+    use Moose;
+    use Types::Capabilities qw( +Enqueueable +Dequeueable );
+    use Types::Common qw( +CodeRef );
+    
+    has queue => (
+      is        => 'ro',
+      isa       => Enqueueable & Dequeueable,
+      default   => sub { [] },
+      handles   => [ qw/ enqueue dequeue / ],
+    );
+    
+    sub run_next ( $self ) {
+      
+      my $task = assert_CodeRef( $self->dequeue or return $self );
+      
+      try {
+        $task->();
+      }
+      catch ( $e ) {
+        warn "Error running task: $e, sending to back of queue";
+        $self->enqueue( $task );
+      }
+      
+      return $self;
+    }
+  }
+  
+  my $q = TaskQueue->new;
+  $q->enqueue( sub { ... } );
+  $q->enqueue( sub { ... } );
+  $q->run_next->run_next->run_next;
+
 =head1 DESCRIPTION
 
 This module allows you to indicate when you are designing your API that you
@@ -261,7 +301,8 @@ result of applying that to all items in that collection. The results of
 calling C<map> in scalar context are not specified, but it may return another
 collection-like object which further operations can be carried out on.
 
-Can be coerced from B<ArrayRef>, B<Greppable>, B<Eachable>, or B<ArrayLike>.
+Can be coerced from B<ArrayRef>, B<Greppable>, B<Eachable>, B<ArrayLike>,
+B<CodeRef>, B<FileHandle>, or B<File>. (Files are read by line.)
 
 =item B<Greppable>
 
@@ -273,7 +314,8 @@ return items in that collection where the coderef returned true. The results of
 calling C<grep> in scalar context are not specified, but it may return another
 collection-like object which further operations can be carried out on.
 
-Can be coerced from B<ArrayRef>, B<Mappable>, B<Eachable>, or B<ArrayLike>.
+Can be coerced from B<ArrayRef>, B<Mappable>, B<Eachable>, B<ArrayLike>,
+B<CodeRef>, B<FileHandle>, or B<File>. (Files are read by line.)
 
 =item B<Sortable>
 
@@ -288,7 +330,7 @@ it may return another collection-like object which further operations can be
 carried out on.
 
 Can be coerced from B<ArrayRef>, B<Mappable>, B<Greppable>, B<Eachable>,
-or B<ArrayLike>.
+B<CodeRef>, B<FileHandle>, or B<File>. (Files are read by line.)
 
 =item B<Reversible>
 
@@ -300,7 +342,7 @@ C<reverse> in scalar context are not specified, but it may return another
 collection-like object which further operations can be carried out on.
 
 Can be coerced from B<ArrayRef>, B<Mappable>, B<Greppable>, B<Eachable>,
-or B<ArrayLike>.
+B<ArrayLike>, B<FileHandle>, or B<File>. (Files are read by line.)
 
 =item B<Countable>
 
@@ -310,7 +352,7 @@ The expectation is that when the method is called in scalar context, it should
 return the number of items in the collection.
 
 Can be coerced from B<ArrayRef>, B<Mappable>, B<Greppable>, B<Eachable>,
-or B<ArrayLike>.
+B<ArrayLike>, B<FileHandle>, or B<File>. (Files are read by line.)
 
 =item B<Joinable>
 
@@ -323,7 +365,7 @@ to use as a separator; if no separator is given, the method may use a default
 separator, typically something like "," or the empty string.
 
 Can be coerced from B<ArrayRef>, B<Mappable>, B<Greppable>, B<Eachable>,
-or B<ArrayLike>.
+B<ArrayLike>, B<FileHandle>, or B<File>. (Files are read by line.)
 
 =item B<Eachable>
 
@@ -332,7 +374,8 @@ An object which provides an C<each> method.
 The expectation is that when the method is called in void context and passed
 a coderef, it should call the coderef for each item in the collection.
 
-Can be coerced from B<ArrayRef>, B<Mappable>, B<Greppable>, or B<ArrayLike>.
+Can be coerced from B<ArrayRef>, B<Mappable>, B<Greppable>, B<ArrayLike>,
+B<CodeRef>, B<FileHandle>, or B<File>. (Files are read by line.).
 
 =back
 
@@ -348,7 +391,12 @@ The expectation is that the method can be called with a single item to add
 that item to the end of the collection.
 
 Can be coerced from B<ArrayRef>, B<Mappable>, B<Greppable>, B<Eachable>,
-or B<ArrayLike>.
+B<ArrayLike>, or B<CodeRef>.
+
+Note that if you coerce a B<Enqueueable> object from a B<ArrayRef>, then
+pushing onto the coerced object will also push onto the original array.
+In theory this should work for B<ArrayLike> too, though it depends on
+how the original array-like object implemented overloading.
 
 =item B<Dequeueable>
 
@@ -358,7 +406,12 @@ The expectation is that when the method is called in a scalar context, it
 will remove an item from the front of the collection and return it.
 
 Can be coerced from B<ArrayRef>, B<Mappable>, B<Greppable>, B<Eachable>,
-or B<ArrayLike>.
+B<ArrayLike>, or B<CodeRef>.
+
+Note that if you coerce a B<Dequeueable> object from a B<ArrayRef>, then
+shifting from the coerced object will also shift from the original array.
+In theory this should work for B<ArrayLike> too, though it depends on
+how the original array-like object implemented overloading.
 
 =item B<Peekable>
 
@@ -391,7 +444,12 @@ that item to the end of the collection. (This behaviour is essentially the
 same as C<enqueue>.)
 
 Can be coerced from B<ArrayRef>, B<Mappable>, B<Greppable>, B<Eachable>,
-or B<ArrayLike>.
+B<ArrayLike>, or B<CodeRef>.
+
+Note that if you coerce a B<Pushable> object from a B<ArrayRef>, then
+pushing onto the coerced object will also push onto the original array.
+In theory this should work for B<ArrayLike> too, though it depends on
+how the original array-like object implemented overloading.
 
 =item B<Poppable>
 
@@ -401,7 +459,12 @@ The expectation is that when the method is called in a scalar context, it
 will remove an item from the end of the collection and return it.
 
 Can be coerced from B<ArrayRef>, B<Mappable>, B<Greppable>, B<Eachable>,
-or B<ArrayLike>.
+B<ArrayLike>, or B<CodeRef>.
+
+Note that if you coerce a B<Pushable> object from a B<ArrayRef>, then
+popping from the coerced object will also pop from the original array.
+In theory this should work for B<ArrayLike> too, though it depends on
+how the original array-like object implemented overloading.
 
 =item B<Peekable>
 
@@ -436,6 +499,101 @@ own class to implement those capabilities.
     coerce  => 1,
   );
 
+=head2 Coercion from B<CodeRef>
+
+Coderefs can be used as iterators to be coerced into B<Eachable>, B<Mappable>,
+and B<Greppable> objects. The C<each>, C<map>, and C<grep> methods will call
+the coderef in list context to return one or more items. The empty list should
+be returned to indicate that the iterator has been exhausted.
+
+Coderefs can also be coerced to C<Dequeueable> objects. The coderef will be
+is expected to dequeue and return a single item.
+
+Coderefs can also be coerced to C<Enqueueable> objects. The coderef will be
+called with a single item to enqueue.
+
+Coderefs can also be coerced to C<Poppable> objects. The coderef will be
+is expected to pop and return a single item.
+
+Coderefs can also be coerced to C<Pushable> objects. The coderef will be
+called with a single item to push.
+
+Here is a simple implementation of a FIFO queue.
+
+  use v5.10;
+  
+  # If an item is on @_, then enqueue it.
+  # Otherwise, it's a dequeue operation.
+  my $q = ( Enqueueable & Dequeueable )->coerce( sub {
+    state $list = [];
+    return push @$list, $_[0] if @_;
+    return if !@$list;
+    shift @$list;
+  } );
+  
+  $q->enqueue( 'first' );
+  $q->enqueue( 'second' );
+  say $q->dequeue;   # first
+  $q->enqueue( 'third' );
+  say $q->dequeue;   # second
+  say $q->dequeue;   # third
+
+=head2 Coercion from B<FileHandle> and B<File>
+
+File handles opened for input can be coerced into B<Mappable>, B<Greppable>,
+B<Sortable>, B<Reversible>, B<Countable>, B<Joinable>, and B<Eachable> objects.
+
+Files are treated as lists of lines using C<< "\n" >> as the line separator.
+Lines are chomped to remove the trailing C<< "\n" >>. C<< seek($fh, 0, 0) >>
+at the start of each method.
+
+L<Path::Tiny> objects representing files will also work. They will be opened
+in UTF-8 mode.
+
+The following variables can tweak how this coercion works, though be wary
+of changing them as it can have global effects.
+
+=over
+
+=item C<< $Types::Capabilities::CoercedValue::FILEHANDLE::INPUT_RECORD_SEPARATOR >>
+
+Defaults to C<< "\n" >>.
+
+=item C<< $Types::Capabilities::CoercedValue::FILEHANDLE::AUTO_SEEK >>
+
+Defaults to true.
+
+=item C<< $Types::Capabilities::CoercedValue::FILEHANDLE::AUTO_CHOMP >>
+
+Defaults to true.
+
+=item C<< $Types::Capabilities::CoercedValue::FILEHANDLE::PATHTINY_FH_OPTIONS >>
+
+Defaults to C<< { locked => 1 } >>.
+
+=item C<< $Types::Capabilities::CoercedValue::FILEHANDLE::PATHTINY_FH_BINMODE >>
+
+Defaults to C<< ":utf8" >>.
+
+=back
+
+Note that the values of these variables at the time of coercion are what
+is important.
+
+  use Types::Capabilities qw( to_Eachable );
+  
+  my $eachable = do {
+    local $Types::Capabilities::CoercedValue::FILEHANDLE::INPUT_RECORD_SEPARATOR = "\r\n";
+    to_Eachable( $fh );
+  };
+  
+  $Types::Capabilities::CoercedValue::FILEHANDLE::INPUT_RECORD_SEPARATOR = "something else";
+  
+  # Iterates through file using "\r\n" as the line ending.
+  $eachable->each( sub {
+    ...;
+  } );
+
 =head1 BUGS
 
 Please report any bugs to
@@ -447,18 +605,23 @@ Largely inspired by: L<Data::Collection>.
 
 L<Sub::HandlesVia>, L<Hydrogen::Autobox>.
 
+B<ArrayRef>, B<CodeRef>, and B<FileHandle> are defined in L<Types::Standard>.
+
+B<ArrayLike> is defined in L<Types::TypeTiny>.
+
+B<File> is defined in L<Types::Path::Tiny>.
+
 =head1 AUTHOR
 
 Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2025 by Toby Inkster.
+This software is copyright (c) 2025-2026 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
-
 =head1 DISCLAIMER OF WARRANTIES
 
 THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
