@@ -5,16 +5,9 @@ use warnings;
 package Types::Capabilities::CoercedValue::QUEUE;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.002001';
+our $VERSION   = '0.003000';
 
 use parent 'Types::Capabilities::CoercedValue::ARRAYREF';
 
-sub new {
-	my $class = shift;
-	my $new = bless [ @{+shift} ], $class;
-	return $new;
-}
-
-1;
-
+__PACKAGE__
 __END__
